import spiderman._web as _web
import spiderman as sp
import numpy as np
import copy 

def get_filter(response):
	filter_wvls = []
	filter_responses = []
	hc = 1.98644582e-25
	for line in open(response,'r'):
		l = line.strip('\n').split(' ')
		wvl = float(l[0])
		filter_wvls += [wvl]
		# dividing by the factor of wvl makes the response proportional 
		# to units of energy density instead of photon number
		filter_responses += [hc*float(l[1]) / wvl]
	filter_wvls = np.array(filter_wvls)
	filter_responses = np.array(filter_responses)
	filter = [filter_wvls,filter_responses]
	return filter

def one_in_one_out(c1,c2,e1,e2,r_inner,r_outer,r2,x2,y2):
	return _web.one_in_one_out(c1,c2,e1,e2,r_inner,r_outer,r2,x2,y2)

def heron(a,b,c):
	return _web.heron(a,b,c)

def find_segment_area(c1x,c2x,r,theta):
	return _web.find_segment_area(c1x,c2x,r,theta)

def find_quad_area(a,b,c,d):
	return _web.find_quad_area(a,b,c,d)

def segment(r,theta):
	return _web.segment(r,theta)

def circle_intersect(x1,y1,r1,x2,y2,r2):
	return _web.circle_intersect(x1,y1,r1,x2,y2,r2)

def line_intersect(x1,y1,x2,y2,r2):
	return _web.line_intersect(x1,y1,x2,y2,r2)

def generate_planet(spider_params,t,use_phase=False,stellar_grid=False,logg=4.5, stellar_model = "blackbody"):
	if use_phase == True:
		t = spider_params.t0 + spider_params.per*t
	brightness_params = spider_params.format_bright_params()

	if spider_params.thermal == True:
		if ((spider_params.brightness_type == 9) or (spider_params.brightness_type == 10)):
			if stellar_grid == False:
				star_grid = sp.stellar_grid.gen_grid(spider_params.l1,spider_params.l2,logg=logg,response=spider_params.filter, stellar_model = stellar_model)
				teffs = star_grid[0]
				totals = star_grid[1]
			else:
				teffs = stellar_grid[0]
				totals = stellar_grid[1]
		else:
			teffs = []
			totals = []
	else:
		teffs = []
		totals = []

	n_star = len(totals)

	spider_params.calc_substellar(t)

	return np.array(_web.generate_planet(spider_params.n_layers,spider_params.lambda0,spider_params.phi0,spider_params.p_u1,spider_params.p_u2,spider_params.brightness_type,brightness_params,teffs,totals,n_star,spider_params.rp,copy.copy(spider_params.grid[0]),copy.copy(spider_params.grid[1]),copy.copy(spider_params.grid[2])))

def call_map_model(spider_params,la,lo):
	# DEPRECATED - NOT CURRENTLY USED BY HIGHER LEVEL FUNCTIONS, NEEDS UPDATING

	brightness_params = spider_params.format_bright_params()
	return np.array(_web.call_map_model(la,lo,spider_params.brightness_type,brightness_params,copy.copy(spider_params.grid[0]),copy.copy(spider_params.grid),copy.copy(spider_params.grid[2])))

def blocked(n_layers,x2,y2,r2):
	return _web.blocked(n_layers,x2,y2,r2)

def zhang_2016(lat,lon,xi,T_n,delta_T):
	return _web.zhang_2016(lat,lon,xi,T_n,delta_T)

def separation_of_centers(t,spider_params):
	ratio = 1/spider_params.rp
	return _web.separation_of_centers(t,spider_params.t0,spider_params.per,spider_params.a_abs,spider_params.inc,spider_params.ecc,spider_params.w,spider_params.a,ratio)

def lightcurve(t,spider_params,stellar_grid=False,logg=4.5,use_phase=False):

	if use_phase == True:
		t = spider_params.t0 + spider_params.per*t

	brightness_params = spider_params.format_bright_params()

	if spider_params.thermal == True:
		if stellar_grid == False:
			star_grid = sp.stellar_grid.gen_grid(spider_params.l1,spider_params.l2,logg=logg,response=spider_params.filter, stellar_model = spider_params.stellar_model)
			teffs = star_grid[0]
			totals = star_grid[1]
		else:
			teffs = stellar_grid[0]
			totals = stellar_grid[1]
	else:
		teffs = []
		totals = []

	if spider_params.filter != False:
		use_filter = 1
		filter = get_filter(spider_params.filter)
	else:
		use_filter = 0
		filter = [[],[]]

	n_wvls = len(filter[0])

	out = _web.lightcurve(spider_params.n_layers,t,spider_params.t0,spider_params.per,spider_params.a_abs,spider_params.inc,spider_params.ecc,spider_params.w,spider_params.a,spider_params.rp,spider_params.p_u1,spider_params.p_u2,spider_params.brightness_type,brightness_params,teffs,totals,len(totals), spider_params.eclipse, filter[0], filter[1], n_wvls,use_filter,copy.copy(spider_params.grid[0]),copy.copy(spider_params.grid[1]),copy.copy(spider_params.grid[2]))

	return np.array(out)

def bb_grid(l1,l2,T_start,T_end,n_temps,n_segments):
	temps, fluxes, deriv = _web.bb_grid(l1,l2,T_start,T_end,n_temps,n_segments)
	return np.array(temps), np.array(fluxes), np.array(deriv)




import spiderman as sp
import numpy as np
import matplotlib.pyplot as plt
import time as timing

def plot_test():
	spider_params = sp.ModelParams(brightness_model='zhang')

	spider_params.n_layers= 20

	spider_params.t0= 200               # Central time of PRIMARY transit [days]
	spider_params.per= 0.81347753       # Period [days]
	spider_params.a_abs= 0.01526        # The absolute value of the semi-major axis [AU]
	spider_params.inc= 82.33            # Inclination [degrees]
	spider_params.ecc= 0.0              # Eccentricity
	spider_params.w= 90                 # Argument of periastron
	spider_params.rp= 0.1594            # Planet to star radius ratio
	spider_params.a= 4.855              # Semi-major axis scaled by stellar radius
	spider_params.p_u1= 0               # Planetary limb darkening parameter
	spider_params.p_u2= 0               # Planetary limb darkening parameter

	spider_params.xi= 0.3       # Ratio of radiative to advective timescale             
	spider_params.T_n= 1128     # Temperature of nightside
	spider_params.delta_T= 942  # Day-night temperature contrast
	spider_params.T_s = 5000    # Temperature of the star

	spider_params.l1 = 1.3e-6    # start of integration channel in microns
	spider_params.l2 = 1.6e-6    # end of integration channel in microns

	t= spider_params.t0 + np.linspace(0, + spider_params.per,100)

	lc = sp.lightcurve(t,spider_params)

	plt.plot(t,lc)
	plt.show()


def time_test(nlayers=5,tpoints=100,nreps=1000):

	spider_params = sp.ModelParams(brightness_model='zhang')

#	spider_params = sp.ModelParams(brightness_model='uniform brightness')

	spider_params.n_layers= nlayers

	spider_params.t0= 200               # Central time of PRIMARY transit [days]
	spider_params.per= 0.81347753       # Period [days]
	spider_params.a_abs= 0.01526        # The absolute value of the semi-major axis [AU]
	spider_params.inc= 82.33            # Inclination [degrees]
	spider_params.ecc= 0.0              # Eccentricity
	spider_params.w= 90                 # Argument of periastron
	spider_params.rp= 0.1594            # Planet to star radius ratio
	spider_params.a= 4.855              # Semi-major axis scaled by stellar radius
	spider_params.p_u1= 0               # Planetary limb darkening parameter
	spider_params.p_u2= 0               # Planetary limb darkening parameter

	spider_params.xi= 0.3       # Ratio of radiative to advective timescale             
	spider_params.T_n= 1128     # Temperature of nightside
	spider_params.delta_T= 942  # Day-night temperature contrast
	spider_params.T_s = 4500    # Temperature of the star

	spider_params.l1 = 1.3e-6    # start of integration channel in microns
	spider_params.l2 = 1.6e-6    # end of integration channel in microns

	spider_params.pb = 0.01    # planet relative brightness

	t= spider_params.t0 + np.linspace(0, + spider_params.per,tpoints)

	print('')
	print('About to generate {} lightcurves with {} layers and {} timepoints'.format(nreps,spider_params.n_layers,tpoints))
	print('')

	start = timing.time()

	star_grid = sp.stellar_grid.gen_grid(spider_params.l1,spider_params.l2)

	ends = []
	for i in range(0,nreps):
		lc = sp.lightcurve(t,spider_params,stellar_grid=star_grid)
		ends += [timing.time()]
	ends = np.array(ends)

	exec_times = np.diff(ends)

	total = ends[-1] - start

	medtime = np.median(exec_times)
	stdtimes = np.std(exec_times)
	medtime = np.median(exec_times)


	print('In total it took {} seconds'.format(round(total,2)))
	print('Each function call was between {:.2E} and {:.2E}seconds'.format(np.min(exec_times),np.max(exec_times)))
	print('Median execution time was {:.2E} seconds'.format(medtime))
	print('Standard deviation was {:.2E} seconds'.format(stdtimes))
	print('{} lightcurves generated per second!'.format(round(1.0/medtime),1))
	print('')



import numpy as np
import spiderman as sp
import spiderman._web as _web
import spiderman.plot as splt
import matplotlib.pyplot as plt
from scipy.optimize import minimize
import math

class MultiModelParams(object):
	def __init__(self,brightness_models=['zhang','lambertian'],**kwargs):
			self.webs = []
			for i in range(0,len(brightness_models)):
				self.webs += [sp.ModelParams(brightness_models[i],**kwargs)]

	def lightcurve(self,t,**kwargs):
		total = sp.lightcurve(t,self.webs[0],**kwargs)
		for i in range(1,len(self.webs)):
			total += sp.lightcurve(t,self.webs[i],**kwargs) - 1.0

		return total


class ModelParams(object):
	def __init__(self,brightness_model='zhang',thermal=False, nearest=None):

		self.n_layers = 5			# The default resolution for the grid

		self.t0= None				# The time of central **PRIMARY** transit [jd]
		self.per= None				# Orbital period of the planet [days]
		self.a_abs= None			# Absolute value of the semi major axis [AU]
		self.inc= None				# Inclination of the planetary orbit (90 is face on) [degrees]
		self.ecc= None				# Eccentricity
		self.w= None				# Longitude of periastron [degrees]
		self.a= None				# Semi major axis, scaled by stellar radius [-]
		self.rp= None				# Planet radius as a fraction of stellar radius [-]
		self.p_u1= None				# **PLANETARY** limb darkening coefficients [-]
		self.p_u2= None				# **PLANETARY** limb darkening coefficients [-]
		self.eclipse = True			# specifies whether to include the drop in flux due to the eclipse
		self.filter = False			# Can use an external response file.
		self.grid = [[],[],[[]]]			# needed in case the "direct read" method is wanted
		self.nearest = nearest         # used for choosing which interpolation model to use - default is spline


		if brightness_model == 'uniform brightness':
			self.n_layers = 1		# The default resolution for the grid

			self.brightness_type= 0	# Integer model identifier
			self.pb= None			# Relative planet brightness (Star is 1)
			self.thermal= False			# Is this a thermal distribution?

		elif brightness_model == 'uniform temperature':
			self.n_layers = 1		# The default resolution for the grid

			self.brightness_type= 1	# Integer model identifier
			self.T_p= None			# Relative planet brightness (Star is 1)
			self.T_s= None			# **STELLAR** effective temperature
			self.thermal= True			# Is this a thermal distribution?

		elif brightness_model == 'two brightness':
			self.brightness_type= 2 # Integer model identifer
			self.pb_d= None			# Relative planet brightness (Star is 1)
			self.pb_n= None			# Relative planet brightness (Star is 1)

			self.thermal= False			# Is this a thermal distribution?
			if not (hasattr(self,'grid_size')):
				self.grid_size = 10


		elif brightness_model == 'two temperature':
			self.brightness_type= 3	# Integer model identifier
			self.pb_d= None			# Relative planet brightness (Star is 1)
			self.pb_n= None			# Relative planet brightness (Star is 1)
			self.T_s= None			# **STELLAR** effective temperature
			self.thermal= True			# Is this a thermal distribution?
			if not (hasattr(self,'grid_size')):
				self.grid_size = 10

		elif brightness_model == 'zhang':
			self.brightness_type= 4	# Integer model identifier
			self.xi= None			# Ratio between radiative and advective timescales
			self.T_n= None			# Radiative solution temperature on night side
			self.delta_T= None		# Day/Night side difference between radiative-only temperature
			self.T_s= None			# **STELLAR** effective temperature
			self.thermal= True			# Is this a thermal distribution?

		elif brightness_model == 'spherical':
			self.a= None			# Ratio between radiative and advective timescales
			self.thermal= thermal			# Is this a thermal distribution?
			if thermal == True:
				self.brightness_type= 14	# Integer model identifier
			else:
				self.brightness_type= 5	# Integer model identifier

                elif brightness_model == 'Louden':
                        self.brightness_type= 6 # Integer model identifer
                        self.insol = None               # insolation in W/m^2
                        self.albedo = None              # albedo
                        self.redist = None              # fraction of incident energy redistributed to the night-side
                        self.T_s = None
                        self.T_int = None

#		elif brightness_model == 'kreidberg':
#			self.brightness_type= 6 # Integer model identifer
#			self.insol = None               # insolation in W/m^2
#			self.albedo = None              # albedo
#			self.redist = None              # fraction of incident energy redistributed to the night-side
#			self.T_s = None

		elif brightness_model == 'hotspot_b':
			self.brightness_type= 7 # Integer model identifer
			self.thermal= False			# Is this a thermal distribution?
			if not (hasattr(self,'grid_size')):
				self.grid_size = 10


		elif brightness_model == 'hotspot_t':
			self.brightness_type= 8 # Integer model identifer
			self.T_s = None
			self.thermal= True			# Is this a thermal distribution?
			if not (hasattr(self,'grid_size')):
				self.grid_size = 10

		elif brightness_model == 'lambertian':
			self.brightness_type= 9 # Integer model identifer
			self.thermal= False			# Is this a thermal distribution?

		elif brightness_model == 'combine':
			self.brightness_type= 10 # Integer model identifer
			self.thermal= True			# Is this a thermal distribution?

		elif brightness_model == 'clouds':
			self.brightness_type= 11 # Integer model identifer
			self.thermal= True			# Is this a thermal distribution?

		elif brightness_model == 'direct_T':
			self.brightness_type= 12 # Integer model identifer
			self.thermal= True			# Is this a thermal distribution?

		elif brightness_model == 'direct_b':
			self.brightness_type= 13 # Integer model identifer
			self.thermal= False			# Is this a thermal distribution?

		else:
			print('Brightness model "'+str(brightness_model)+'" not recognised!')
			quit()

	def format_bright_params(self):
		if (self.brightness_type == 0):
			brightness_param_names = ['pb']
			try:
				brightness_params = [self.pb]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()
		elif (self.brightness_type == 1):
			brightness_param_names = ['T_s','l1','l2','T_p']
			try:
				brightness_params = [self.T_s,self.l1,self.l2,self.T_p]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()
		elif (self.brightness_type == 2):
			brightness_param_names = ['pb_d','pb_n']
#			try:
#				brightness_params = [self.pb_d,self.pb_n]

#			self.brightness_type = 7
			try:
				brightness_params = [0, 0, self.pb_n, self.grid_size,self.pb_d, 90]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()
		elif (self.brightness_type == 3):
			brightness_param_names = ['T_s','l1','l2','T_p_d','T_p_n']
#			try:
#				brightness_params = [self.T_s,self.l1,self.l2,self.T_p_d,self.T_p_n]
#			self.brightness_type = 8
			try:
				brightness_params = [self.T_s,self.l1, self.l2, self.grid_size, 0, 0, self.T_p_n, self.T_p_d, 90]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()
		elif (self.brightness_type == 4):
			brightness_param_names = ['T_s','l1','l2','xi','T_n','delta_T']
			try:
				brightness_params = [self.T_s,self.l1,self.l2,self.xi,self.T_n,self.delta_T]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()

		elif (self.brightness_type == 5 or self.brightness_type == 14):

			brightness_param_names = ['degree','sph','la0','lo0']
			if self.brightness_type == 14:
				brightness_param_names = ['T_s','l1','l2'] + brightness_param_names


			brightness_params = [self.degree,self.la0,self.lo0] + self.sph
			if self.brightness_type == 14:
				brightness_params = [self.T_s,self.l1,self.l2] + brightness_params

			try:
				brightness_params = [self.degree,self.la0,self.lo0] + self.sph
				if self.brightness_type == 14:
					brightness_params = [self.T_s,self.l1,self.l2] + brightness_params
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()
#			total_modes = (self.degree * (self.degree +1))/2.0
			total_modes = (self.degree)**2
			if len(self.sph) != total_modes:
				print('You have not specified the correct number of mode coefficients!')
				print('You gave '+str(int(len(self.sph)))+', there should be '+str(int(total_modes)))
				quit()

#                elif (self.brightness_type == 6):
#                        brightness_param_names = ['T_s','l1','l2','insol','albedo','redist']
#                        try:
#                                brightness_params = [self.T_s, self.l1, self.l2, self.insol, self.albedo, self.redist]
#                        except:
#                                print('Brightness parameters incorrectly assigned')
#                                print('should be',brightness_param_names)
#                                quit()

		elif (self.brightness_type == 6):
			brightness_param_names = ['T_s','l1','l2','insol','albedo','redist','T_int']
			try:
				brightness_params = [self.T_s, self.l1, self.l2, self.insol, self.albedo, self.redist,self.T_int]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()

		elif (self.brightness_type == 7):
			brightness_param_names = ['la0','lo0','p_b','spot_b','size']
			try:
				brightness_params = [self.la0, self.lo0, self.p_b, self.grid_size,self.spot_b, self.size]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()

		elif (self.brightness_type == 8):
			brightness_param_names = ['T_s','l1','l2','la0','lo0','p_T','spot_T','size']
			try:
				brightness_params = [self.T_s,self.l1, self.l2, self.grid_size, self.la0, self.lo0, self.p_T, self.spot_T, self.size]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()

		elif (self.brightness_type == 9):
			brightness_param_names = ['albedo']
			ars = 1.0/self.a
			r2 = 1.0/self.rp

			if not hasattr(self, 'T_s'):
			    self.T_s = 0
			    self.l1 = 0
			    self.l2 = 0

			try:
				brightness_params = [self.T_s,self.l1,self.l2,self.albedo,ars,r2]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()
		elif (self.brightness_type == 10):
			brightness_param_names = ['T_s','l1','l2','xi','T_n','delta_T','albedo']
			ars = 1.0/self.a
			r2 = 1.0/self.rp
			try:
				brightness_params = [self.T_s,self.l1,self.l2,self.xi,self.T_n,self.delta_T,self.albedo,ars,r2]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()

		elif (self.brightness_type == 11):
			brightness_param_names = ['T_s','l1','l2','xi','T_n','delta_T','cloud']
			try:
				brightness_params = [self.T_s,self.l1,self.l2,self.xi,self.T_n,self.delta_T,self.clouds]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()
		elif (self.brightness_type == 12):
			brightness_param_names = ['T_s','l1','l2','grid']
			n_lo = len(self.grid[0])
			n_la = len(self.grid[1])

#			chooses whether to use a 2d spline or a nearest neighbor method. default is spline.
			if self.nearest == None:
				nearest = 0
			elif self.nearest == True:
				nearest = 1 
			elif self.nearest == False:
				nearest = 0

			try:
				brightness_params = [self.T_s,self.l1,self.l2,n_lo,n_la,nearest]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()

		elif (self.brightness_type == 13):
			brightness_param_names = ['grid']
			n_lo = len(self.grid[0])
			n_la = len(self.grid[1])

#			chooses whether to use a 2d spline or a nearest neighbor method. default is spline.
			if self.nearest == None:
				nearest = 0
			elif self.nearest == True:
				nearest = 1 
			elif self.nearest == False:
				nearest = 0

			try:
				brightness_params = [n_lo,n_la,nearest]
			except:
				print('Brightness parameters incorrectly assigned')
				print('should be',brightness_param_names)
				quit()


		elif any(b == None for b in brightness_params):
			print('Brightness parameters incorrectly assigned')
			print('should be',brightness_param_names)
			quit()
		return brightness_params

	def plot_square(self,*args,**kwargs):
		return splt.plot_square(self,*args,**kwargs)

	def plot_system(self,*args,**kwargs):
		return splt.plot_system(self,*args,**kwargs)

	def plot_planet(self,*args,**kwargs):
		return splt.plot_planet(self,*args,**kwargs)

	def plot_quad(self,*args,**kwargs):
		return splt.plot_quad(self,*args,**kwargs)

	def plot_uncertainty(self,*args,**kwargs):
		return splt.plot_uncertainty(self,*args,**kwargs)

	def lightcurve(self,t,**kwargs):
		return sp.lightcurve(t,self,**kwargs)

	def calc_phase(self,t):
		self.phase = _web.calc_phase(t,self.t0,self.per)

	def calc_substellar(self,t):
		self.calc_phase(t)
		coords = sp.separation_of_centers(t,self)
		substellar = _web.calc_substellar(self.phase,np.array(coords))
		self.lambda0 = substellar[0]
		self.phi0 = substellar[1]

	def get_lims(self,t,temp_map=False,use_phase=False):
		if use_phase == True:
			if self.t0 == None:
				self.t0 = 0.0
			if self.per == None:
				self.per = 1.0
			t = self.t0 + self.per*t

		planet = sp.generate_planet(self,t)
		if temp_map == True:
			b_i = 17
		else:
			b_i = 16

		temps = planet[:,b_i]

		return [np.min(temps),np.max(temps)]

	def eclipse_depth(self,phase=0.5,stellar_grid=False):

		brightness_params = self.format_bright_params()

		if self.thermal == True:
			if stellar_grid == False:
				star_grid = sp.stellar_grid.gen_grid(self.l1,self.l2,logg=4.5,response=self.filter)
				teffs = star_grid[0]
				totals = star_grid[1]
			else:
				teffs = stellar_grid[0]
				totals = stellar_grid[1]
		else:
			teffs = []
			totals = []

		t = 0.0 + np.array([phase])

		if(self.inc == None):
			self.inc = 90.0

		if(self.p_u1 == None):
			self.p_u1 = 0.0

		if(self.p_u2 == None):
			self.p_u2 = 0.0

		if(self.a == None):
			self.a = 4.0

		if(self.w == None):
			self.w = 0.0

		if(self.ecc == None):
			self.ecc = 0.0


		if(self.a_abs == None):
			self.a_abs = 1.0

		if self.filter != False:
			use_filter = 1
			filter = get_filter(self.filter)
		else:
			use_filter = 0
			filter = [[],[]]

		n_wvls = len(filter[0])


		out = _web.lightcurve(self.n_layers,t,0.0,1.0,self.a_abs,self.inc,0.0,0.0,self.a,self.rp,self.p_u1,self.p_u2,self.brightness_type,brightness_params,teffs,totals,len(totals),0, filter[0], filter[1], n_wvls,use_filter,self.grid[0],self.grid[1],self.grid[2])[0] - _web.lightcurve(self.n_layers,t,0.0,1.0,self.a_abs,self.inc,0.0,0.0,self.a,self.rp,self.p_u1,self.p_u2,self.brightness_type,brightness_params,teffs,totals,len(totals),1, filter[0], filter[1], n_wvls,use_filter,self.grid[0],self.grid[1],self.grid[2])[0]

		return np.array(out)

	def avg_day(self,**kwargs):
		return self.avg_region([-np.pi/2,np.pi/2],[-np.pi/2,np.pi/2],**kwargs)

	def avg_night(self,**kwargs):
		return self.avg_region([-np.pi/2,np.pi/2],[-np.pi/2 - np.pi,np.pi/2  - np.pi],**kwargs)

	def avg_planet(self,**kwargs):
		return self.avg_region([-np.pi/2,np.pi/2],[-np.pi,np.pi],**kwargs)

	def avg_region(self,la_lim,lo_lim,nla=100,nlo=100,stellar_grid=False,reflection=False,temp_map=True):

		las = np.linspace(la_lim[0],la_lim[1],nla)
		los = np.linspace(lo_lim[0],lo_lim[1],nlo)

		for i in range(0,len(los)):
			if los[i] > np.pi:
				los[i] -= 2*np.pi
			if los[i] < -np.pi:
				los[i] += 2*np.pi

		tot_flux = 0.0
		tot_weight = 0.0

		for la in las:
			weight = np.cos(la)
			for lo in los:
				flux = sp.call_map_model(self,la,lo)
				if temp_map == False:
					tot_flux += weight*flux[0]
				else:
					tot_flux += weight*flux[1]
				tot_weight += weight
		avg = tot_flux/tot_weight

		return avg

	def max_temp(self,**kwargs):
		return(self.find_temp([0.1,0.0],-1,**kwargs))

	def min_temp(self,**kwargs):
		return(self.find_temp([0.1,np.pi],1,**kwargs))

	def find_temp(self,p0,sign,stellar_grid=False,reflection=False,temp_map=True):

		out = minimize(self.find_func,p0,method='L-BFGS-B',bounds=((-np.pi/2,np.pi/2),(None,None)),args=(sign,temp_map))

		p = out['x']

		if p[1] > np.pi:
			revs = math.ceil(abs(p[1]/(2*np.pi)))
			p[1] -= revs*2*np.pi
		if p[1] < -np.pi:
			revs = (math.ceil(abs(p[1]/(2*np.pi))))-1
			p[1] += revs*2*np.pi

		flux = sp.call_map_model(self,p[0],p[1])

		if temp_map == True:
			f = flux[1]
		else:
			f = flux[0]

		return(f,out['x']*180/np.pi)

	def find_func(self,x,sign,temp_map):

		if x[1] > np.pi:
			revs = math.ceil(abs(x[1]/(2*np.pi)))
			x[1] -= revs*2*np.pi
		if x[1] < -np.pi:
			revs = (math.ceil(abs(x[1]/(2*np.pi))))-1
			x[1] += revs*2*np.pi

		flux = sp.call_map_model(self,x[0],x[1])

		if temp_map == True:
			return sign*flux[1]
		else:
			return sign*flux[0]


	def total_luminosity(self,planet_radius,stellar_grid=False,reflection=False):
		p1,p2 = self.phase_brightness([0,0.5],stellar_grid=stellar_grid,reflection=reflection,planet_radius=planet_radius)
		return(p1+p2)


	def phase_brightness(self,phases,stellar_grid=False,reflection=False,planet_radius=False):

		if self.thermal == True:
			if stellar_grid == False:
				stellar_grid = sp.stellar_grid.gen_grid(self.l1,self.l2,logg=4.5,response=self.filter)
				teffs = stellar_grid[0]
				totals = stellar_grid[1]
			else:
				teffs = stellar_grid[0]
				totals = stellar_grid[1]
		else:
			teffs = []
			totals = []

		if type(phases) is not list: phases = [phases]

		brightness_params = self.format_bright_params()

		out_list = []
		for phase in phases:

			t = 0.0 + np.array([phase])

			if(self.t0 == None):
				self.t0 = 0.0

			if(self.per == None):
				self.per = 1.0

			if(self.inc == None):
				self.inc = 90.0

			if(self.p_u1 == None):
				self.p_u1 = 0.0

			if(self.p_u2 == None):
				self.p_u2 = 0.0

			if(self.a == None):
				self.a = 4.0

			if(self.w == None):
				self.w = 0.0

			if(self.ecc == None):
				self.ecc = 0.0


			if(self.a_abs == None):
				self.a_abs = 1.0


			planet = sp.generate_planet(self,t,stellar_grid=stellar_grid)

			brights = planet[:,16]
			areas = planet[:,15]
			inner = planet[:,13]
			outer = planet[:,14]

			avg_dist = ((inner + outer)/2)*np.pi/2
			avg_dist[0] = 0.0

			norm = np.sqrt(np.cos(avg_dist))

			min_bright = np.min(brights)
			max_bright = np.max(brights)

			if planet_radius == False:
				out = np.sum(brights*areas)/np.pi
			else:
				out = np.sum(brights*areas/norm)*planet_radius**2

			out_list += [out]

#			out = _web.lightcurve(self.n_layers,t,0.0,1.0,self.a_abs,self.inc,0.0,0.0,self.a,self.rp,self.p_u1,self.p_u2,self.brightness_type,brightness_params,teffs,totals,len(totals),0)[0] - 1.0
#			out_list += [out]


		if len(out_list) == 1:
			return out_list[0]

#		returns the total flux of the planet at each phase in the defined bandpass in Watts / m^2 / sr

		return out_list



# -*- coding: utf-8 -*-
from os.path import expanduser,join
home = expanduser("~")
param_file = join(home,'.spidermanrc')

class RcParams(object):
	def __init__(self):
		self.dict = {}
		try:
			print('looking for spidermanrc file at '+param_file)
			for line in open(param_file):
				key = line.split(':')[0].replace(' ','')
				cval = line.split(':')[1].replace(' ','').replace('\n','')
				self.dict[key] = cval
			RcParams.read = True
		except:
			RcParams.read = False
			print('no spidermanrc file detected')
	def __getitem__(self, key):
	    return self.dict[key]
rcParams = RcParams()

from spiderman.params import *
from spiderman.web import *
from spiderman.plot import *
from spiderman.test import *
from spiderman.stellar_grid import *
from spiderman.read_model import *

__all__ = ["web","params","_web","plot","test","stellar_grid","read_model"]



import matplotlib as mpl
#mpl.use('Agg')
import spiderman as sp
import numpy as np
import matplotlib.pyplot as plt
from matplotlib._png import read_png
from matplotlib.offsetbox import TextArea, DrawingArea, OffsetImage, \
    AnnotationBbox
import matplotlib.patches as patches
from matplotlib import gridspec
#import seaborn as sb
from numpy.random import randn
from mpl_toolkits.axes_grid1 import make_axes_locatable
import os.path
import batman

def get_star_png():
	spiderdir = os.path.dirname(sp.__file__)
	png_name = os.path.join(spiderdir,'art/sun_transp.png')
	image = read_png(png_name)
	return image

def plot_system(spider_params,t,ax=False,min_temp=False,max_temp=False,temp_map=False,min_bright=0.2,use_phase=False,show_cax=True,mycmap=plt.cm.inferno,theme='white',show_axes=True):

	if theme == 'black':
		bg = 'black'
		tc = ("#04d9ff")
	else:
		bg = 'white'
		tc = 'black'

	if use_phase == True:
		t = spider_params.t0 + spider_params.per*t

	if ax == False:
		f, ax = plt.subplots(facecolor=bg)
	image = sp.get_star_png()
	star_r_pix = 180
	star_offset_pix = 200

	p_imrat = star_r_pix*spider_params.rp

	coords = sp.separation_of_centers(t,spider_params)

	planet_pix = [star_offset_pix - coords[0]*p_imrat,star_offset_pix - coords[1]*p_imrat]

	ax = sp.plot_planet(spider_params,t,ax=ax,min_temp=min_temp,max_temp=max_temp,temp_map=temp_map,min_bright=min_bright,scale_planet=p_imrat,planet_cen=planet_pix,mycmap=mycmap,show_cax=show_cax,theme=theme,show_axes=show_axes)

	if(abs(abs(spider_params.phase)-0.5) > 0.25):
		#in front 
		s_zorder = 1
	else:
		s_zorder = 3

	im = ax.imshow(image,zorder=s_zorder)
	patch = patches.Circle((200, 200), radius=200, transform=ax.transData)
	im.set_clip_path(patch)

	ax.set_xlim(star_offset_pix+40*p_imrat,star_offset_pix+-40*p_imrat)
	ax.set_ylim(star_offset_pix+-10*p_imrat,star_offset_pix+10*p_imrat)

	ax.set(aspect=1)

	arrr = np.array([-1000,-500,0,500,1000])
	arrr2 = np.array([-200,0,200])

	ax.set_xticks(arrr+star_offset_pix)
	ax.set_xticklabels([round(x,3) for x in -1*arrr/(star_r_pix*spider_params.a/spider_params.a_abs)])

	ax.set_yticks(arrr2+star_offset_pix)
	ax.set_yticklabels([round(x,3) for x in 1*arrr2/(star_r_pix*spider_params.a/spider_params.a_abs)])

	ax.set_xlabel('Projected separation (AU)')

	return ax

def plot_quad(spider_params,min_temp=False,max_temp=False,temp_map=False,min_bright=0.2,scale_planet=1.0,planet_cen=[0.0,0.0],use_phase=False,show_cax=True,mycmap=plt.cm.inferno,theme='white'):


		if theme == 'black':
			bg = 'black'
			tc = ("#04d9ff")
		else:
			bg = 'white'
			tc = 'black'

		fig, axs = plt.subplots(2,2,figsize=(8/0.865,8),facecolor=bg)

		# need a "get max" or "get min" function or something similar so the scale can be set properly

		if max_temp == False:
			blims1 = spider_params.get_lims(0,temp_map=temp_map,use_phase=True)
			blims2 = spider_params.get_lims(0.25,temp_map=temp_map,use_phase=True)
			blims3 = spider_params.get_lims(0.5,temp_map=temp_map,use_phase=True)
			blims4 = spider_params.get_lims(0.75,temp_map=temp_map,use_phase=True)
			min_temp = 0.9*np.min(np.array([blims1,blims2,blims3,blims4]))
			max_temp = 1.1*np.max(np.array([blims1,blims2,blims3,blims4]))
			if min_temp == 0.0:
				min_temp = 1e-19

		dp = ((max_temp-min_temp)*min_bright)


		spider_params.plot_planet(0,use_phase=True,ax=axs[0,0],show_cax=False,min_temp=min_temp,max_temp=max_temp,temp_map=temp_map,mycmap=mycmap,theme=theme,min_bright=min_bright)
		spider_params.plot_planet(0.25,use_phase=True,ax=axs[0,1],show_cax=False,min_temp=min_temp,max_temp=max_temp,temp_map=temp_map,mycmap=mycmap,theme=theme,min_bright=min_bright)
		spider_params.plot_planet(0.5,use_phase=True,ax=axs[1,0],show_cax=False,min_temp=min_temp,max_temp=max_temp,temp_map=temp_map,mycmap=mycmap,theme=theme,min_bright=min_bright)
		spider_params.plot_planet(0.75,use_phase=True,ax=axs[1,1],show_cax=False,min_temp=min_temp,max_temp=max_temp,temp_map=temp_map,mycmap=mycmap,theme=theme,min_bright=min_bright)

#		divider = make_axes_locatable(fig)

#		min_val = (dp + min_temp-min_temp)/(dp + max_temp-min_temp)
#		max_val = (dp + max_temp-min_temp)/(dp + max_temp-min_temp)

#		zero_temp = min_temp - dp

		zero_temp = min_temp


		if temp_map == True:
			data = [np.linspace(zero_temp,max_temp,1000)]*2
		else:
			data = [np.linspace(min_temp/max_temp,max_temp/max_temp,1000)]*2
		fake, fake_ax = plt.subplots()
		mycax = fake_ax.imshow(data, interpolation='none', cmap=mycmap)
		plt.close(fake)

		fig.subplots_adjust(right=0.80)
		cbar_ax = fig.add_axes([0.85, 0.15, 0.05, 0.7])

		if show_cax == True:
#			cax = divider.append_axes("right", size="20%", pad=0.05)
		#	cbar = plt.colorbar(mycax, cax=cax,ticks=[1100,1300,1500,1700,1900])
			cbar = plt.colorbar(mycax, cax=cbar_ax)
			cbar.ax.tick_params(colors=tc)

			if temp_map == True:
				cbar.set_label('T (K)',color=tc)  # horizontal colorbar
			else:
				cbar.set_label('Relative brightness',color=tc)  # horizontal colorbar

		fig.subplots_adjust(wspace=0, hspace=0)

		return fig


def plot_uncertainty(spider_params,fs,min_temp=False,max_temp=False,temp_map=True,min_bright=0.2,scale_planet=1.0,planet_cen=[0.0,0.0],use_phase=False,show_cax=True,mycmap=plt.cm.viridis_r,theme='white'):

		if theme == 'black':
			bg = 'black'
			tc = ("#04d9ff")
		else:
			bg = 'white'
			tc = 'black'

		fig, axs = plt.subplots(2,2,figsize=(8/0.865,8),facecolor=bg)

		# need a "get max" or "get min" function or something similar so the scale can be set properly

		fs = np.array(fs)

		min_temp = np.min(fs)
		max_temp = np.max(fs)

		dp = ((max_temp-min_temp)*min_bright)

		sp.plot_dist(spider_params,fs[0],ax=axs[0,0],show_cax=False,min_temp=min_temp,max_temp=max_temp,temp_map=temp_map,mycmap=mycmap,theme=theme,min_bright=min_bright)
		sp.plot_dist(spider_params,fs[1],ax=axs[0,1],show_cax=False,min_temp=min_temp,max_temp=max_temp,temp_map=temp_map,mycmap=mycmap,theme=theme,min_bright=min_bright)
		sp.plot_dist(spider_params,fs[2],ax=axs[1,0],show_cax=False,min_temp=min_temp,max_temp=max_temp,temp_map=temp_map,mycmap=mycmap,theme=theme,min_bright=min_bright)
		sp.plot_dist(spider_params,fs[3],ax=axs[1,1],show_cax=False,min_temp=min_temp,max_temp=max_temp,temp_map=temp_map,mycmap=mycmap,theme=theme,min_bright=min_bright)

#		divider = make_axes_locatable(fig)

#		zero_temp = min_temp - dp
		zero_temp = min_temp

		data = [np.linspace(zero_temp,max_temp,1000)]*2
		fake, fake_ax = plt.subplots()
		mycax = fake_ax.imshow(data, interpolation='none', cmap=mycmap)
		plt.close(fake)

		fig.subplots_adjust(right=0.80)
		cbar_ax = fig.add_axes([0.85, 0.15, 0.05, 0.7])

		if show_cax == True:
#			cax = divider.append_axes("right", size="20%", pad=0.05)
		#	cbar = plt.colorbar(mycax, cax=cax,ticks=[1100,1300,1500,1700,1900])
			cbar = plt.colorbar(mycax, cax=cbar_ax)
			cbar.ax.tick_params(colors=tc)

			cbar.ax.invert_yaxis()

			if temp_map == True:
				cbar.set_label(r'Temperature precision (\%)',color=tc)  # horizontal colorbar
			else:
				cbar.set_label(r'Flux precision (\%)',color=tc)  # horizontal colorbar

		fig.subplots_adjust(wspace=0, hspace=0)

		return fig

def plot_dist(spider_params,temps,ax=False,min_temp=False,max_temp=False,temp_map=False,min_bright=0.2,scale_planet=1.0,planet_cen=[0.0,0.0],mycmap=plt.get_cmap('viridis_r'),show_cax=True,theme='black',show_axes=False):

	if theme == 'black':
		bg = 'black'
		tc = ("#04d9ff")
	else:
		bg = 'white'
		tc = 'black'

	if ax == False:
		f, ax = plt.subplots(facecolor=bg)
		new_ax = True
	else:
		new_ax = False

	planet = sp.generate_planet(spider_params,0,use_phase=True)

	if min_temp == False:
		min_temp = np.min(temps)
		max_temp = np.max(temps)


	dp = ((max_temp-min_temp)*min_bright)

	for j in range (0,len(planet)):
		val = (dp + temps[j]-min_temp)/(dp + max_temp-min_temp)
		c = mycmap(val)

		n = planet[j]

		r1 = n[13]*scale_planet
		r2 = n[14]*scale_planet
		radii = [r1,r2]

		thetas = np.linspace(n[10],n[11],100)

		xs = np.outer(radii, np.cos(thetas)) + planet_cen[0]
		ys = np.outer(radii, np.sin(thetas)) + planet_cen[1]

		xs[1,:] = xs[1,::-1]
		ys[1,:] = ys[1,::-1]

		ax.fill(np.ravel(xs), np.ravel(ys), edgecolor=c,color=c,zorder=2)


	ax.set_facecolor(bg)

	if show_axes == False:
		ax.spines['bottom'].set_color(bg)
		ax.spines['left'].set_color(bg)
		ax.spines['top'].set_color(bg)
		ax.spines['right'].set_color(bg)
	else:
		ax.spines['bottom'].set_color(tc)
		ax.spines['left'].set_color(tc)
		ax.spines['top'].set_color(tc)
		ax.spines['right'].set_color(tc)

	ax.set(aspect=1)

	bs = 1.1
	ax.set_xlim(+bs,-bs)
	ax.set_ylim(-bs,+bs)

	if new_ax == True:
		bs = 1.1
		ax.set_xlim(+bs,-bs)
		ax.set_ylim(-bs,+bs)

	if show_axes == False:
		ax.get_xaxis().set_visible(False)
		ax.get_yaxis().set_visible(False)
	else:
		ax.get_xaxis().set_visible(True)
		ax.get_yaxis().set_visible(True)

	divider = make_axes_locatable(ax)
	# Append axes to the right of ax, with 20% width of ax

#	zero_temp = min_temp - dp
	zero_temp = min_temp
	data = [np.linspace(zero_temp,max_temp,1000)]*2
	fake, fake_ax = plt.subplots()
	mycax = fake_ax.imshow(data, interpolation='none', cmap=mycmap)
	plt.close(fake)

	if show_cax == True:
		cax = divider.append_axes("right", size="20%", pad=0.05)
	#	cbar = plt.colorbar(mycax, cax=cax,ticks=[1100,1300,1500,1700,1900])
		cbar = plt.colorbar(mycax, cax=cax)
		cbar.ax.tick_params(colors=tc)
		cbar.ax.invert_yaxis()

		if temp_map == True:
			cbar.set_label(r'Temperature precision (\%)',color=tc)  # horizontal colorbar
		else:
			cbar.set_label(r'Flux precision (\%)',color=tc)  # horizontal colorbar

	return ax

def plot_defined_planet(planet,ax=False,min_temp=False,max_temp=False,temp_map=False,min_bright=0.2,scale_planet=1.0,planet_cen=[0.0,0.0],mycmap=plt.get_cmap('inferno'),show_cax=True,theme='black',show_axes=False):

	if theme == 'black':
		bg = 'black'
		tc = ("#04d9ff")
	else:
		bg = 'white'
		tc = 'black'

	if ax == False:
		f, ax = plt.subplots(facecolor=bg)
		new_ax = True
	else:
		new_ax = False

	if temp_map == True:
		b_i = 17
	else:
		b_i = 16

	if min_temp == False:
		temps = planet[:,b_i]
		min_temp = np.min(temps)
		max_temp = np.max(temps)


	dp = ((max_temp-min_temp)*min_bright)

	for j in range (0,len(planet)):
		val = (dp + planet[j][b_i]-min_temp)/(dp + max_temp-min_temp)
		c = mycmap(val)

		n = planet[j]

		r1 = n[13]*scale_planet
		r2 = n[14]*scale_planet
		radii = [r1,r2]

		thetas = np.linspace(n[10],n[11],100)

		xs = np.outer(radii, np.cos(thetas)) + planet_cen[0]
		ys = np.outer(radii, np.sin(thetas)) + planet_cen[1]

		xs[1,:] = xs[1,::-1]
		ys[1,:] = ys[1,::-1]

		ax.fill(np.ravel(xs), np.ravel(ys), edgecolor=c,color=c,zorder=2)


	ax.set_facecolor(bg)

	if show_axes == False:
		ax.spines['bottom'].set_color(bg)
		ax.spines['left'].set_color(bg)
		ax.spines['top'].set_color(bg)
		ax.spines['right'].set_color(bg)
	else:
		ax.spines['bottom'].set_color(tc)
		ax.spines['left'].set_color(tc)
		ax.spines['top'].set_color(tc)
		ax.spines['right'].set_color(tc)

	ax.set(aspect=1)

	bs = 1.1
	ax.set_xlim(+bs,-bs)
	ax.set_ylim(-bs,+bs)

	if new_ax == True:
		bs = 1.1
		ax.set_xlim(+bs,-bs)
		ax.set_ylim(-bs,+bs)

	if show_axes == False:
		ax.get_xaxis().set_visible(False)
		ax.get_yaxis().set_visible(False)
	else:
		ax.get_xaxis().set_visible(True)
		ax.get_yaxis().set_visible(True)

	divider = make_axes_locatable(ax)
	# Append axes to the right of ax, with 20% width of ax

#	zero_temp = min_temp - dp
	zero_temp = min_temp
	data = [np.linspace(zero_temp,max_temp,1000)]*2
	fake, fake_ax = plt.subplots()
	mycax = fake_ax.imshow(data, interpolation='none', cmap=mycmap)
	plt.close(fake)

	if show_cax == True:
		cax = divider.append_axes("right", size="20%", pad=0.05)
	#	cbar = plt.colorbar(mycax, cax=cax,ticks=[1100,1300,1500,1700,1900])
		cbar = plt.colorbar(mycax, cax=cax)
		cbar.ax.tick_params(colors=tc)

		if temp_map == True:
			cbar.set_label('T (K)',color=tc)  # horizontal colorbar
		else:
			cbar.set_label('Relative brightness',color=tc)  # horizontal colorbar

	return ax

def plot_square(spider_params,ax=False,min_temp=False,max_temp=False,temp_map=False,min_bright=0.2,scale_planet=1.0,planet_cen=[0.0,0.0],mycmap=plt.get_cmap('inferno'),show_cax=True,theme='white',show_axes=True,nla=100,nlo=100):

	if theme == 'black':
		bg = 'black'
		tc = ("#04d9ff")
	else:
		bg = 'white'
		tc = 'black'

	if ax == False:
		f, ax = plt.subplots(facecolor=bg)
		new_ax = True
	else:
		new_ax = False

	las = np.linspace(-np.pi/2,np.pi/2,nla)
	los = np.linspace(-np.pi,np.pi,nlo)

	plt.ylim(-90,90)
	plt.xlim(-180,180)

	fluxes = []
	for la in las:
		row = []
		for lo in los:
			flux = sp.call_map_model(spider_params,la,lo)
			if temp_map == False:
				row += [flux[0]]
			else:
				row += [flux[1]]
		fluxes += [row]
	fluxes = np.array(fluxes)

	if temp_map == False:
		fluxes = fluxes/np.max(fluxes)

	lala, lolo = np.meshgrid(los,las)

	plt.plot([0],[0],'x',color=('#0cff0c'),ms=10,mew=2)

	ax.set_xlabel('longitude',color=tc)
	ax.set_ylabel('latitude',color=tc)

	ax.set_facecolor(bg)
	if show_axes == False:
		ax.spines['bottom'].set_color(bg)
		ax.spines['left'].set_color(bg)
		ax.spines['top'].set_color(bg)
		ax.spines['right'].set_color(bg)
	else:
		ax.spines['bottom'].set_color(tc)
		ax.spines['left'].set_color(tc)
		ax.spines['top'].set_color(tc)
		ax.spines['right'].set_color(tc)

	if show_axes == False:
		ax.get_xaxis().set_visible(False)
		ax.get_yaxis().set_visible(False)
	else:
		ax.get_xaxis().set_visible(True)
		ax.get_yaxis().set_visible(True)

	ax.tick_params(colors=tc)

	if show_cax == True:
		cax = plt.pcolor(lala*180/np.pi,lolo*180/np.pi,fluxes,cmap=mycmap)
	#	cbar = plt.colorbar(mycax, cax=cax,ticks=[1100,1300,1500,1700,1900])
		cbar = plt.colorbar()
		cbar.ax.tick_params(colors=tc)

		if temp_map == True:
			cbar.set_label('T (K)',color=tc)  # horizontal colorbar
		else:
			cbar.set_label('Relative brightness',color=tc)  # horizontal colorbar

	return ax

	if temp_map == True:
		b_i = 17
	else:
		b_i = 16

	if min_temp == False:
		temps = planet[:,b_i]
		min_temp = np.min(temps)
		max_temp = np.max(temps)


	dp = ((max_temp-min_temp)*min_bright)

	for j in range (0,len(planet)):
		val = (dp + planet[j][b_i]-min_temp)/(dp + max_temp-min_temp)
		c = mycmap(val)

		n = planet[j]

		r1 = n[13]*scale_planet
		r2 = n[14]*scale_planet
		radii = [r1,r2]

		thetas = np.linspace(n[10],n[11],100)

		xs = np.outer(radii, np.cos(thetas)) + planet_cen[0]
		ys = np.outer(radii, np.sin(thetas)) + planet_cen[1]

		xs[1,:] = xs[1,::-1]
		ys[1,:] = ys[1,::-1]

		ax.fill(np.ravel(xs), np.ravel(ys), edgecolor=c,color=c,zorder=2)

	divider = make_axes_locatable(ax)
	# Append axes to the right of ax, with 20% width of ax

#	zero_temp = min_temp - dp
	zero_temp = min_temp
	data = [np.linspace(zero_temp,max_temp,1000)]*2
	fake, fake_ax = plt.subplots()
	mycax = fake_ax.imshow(data, interpolation='none', cmap=mycmap)
	plt.close(fake)

	return ax

def plot_planet(spider_params,t,ax=False,min_temp=False,max_temp=False,temp_map=False,min_bright=0.2,scale_planet=1.0,planet_cen=[0.0,0.0],use_phase=False,show_cax=True,mycmap=plt.cm.inferno,theme='white',show_axes=False):

	if theme == 'black':
		bg = 'black'
		tc = ("#04d9ff")
	else:
		bg = 'white'
		tc = 'black'

	if use_phase == True:
		t = spider_params.t0 + spider_params.per*t

	if ax == False:
		f, ax = plt.subplots(facecolor=bg)
		new_ax = True
	else:
		new_ax = False

	planet = sp.generate_planet(spider_params,t)

	if temp_map == True:
		b_i = 17
	else:
		b_i = 16

	if min_temp == False:
		temps = planet[:,b_i]
		min_temp = np.min(temps)*0.9
		max_temp = np.max(temps)*1.1

	temps = planet[:,b_i]


	dp = ((max_temp-min_temp)*min_bright)


	if((max_temp - min_temp) < 1e-18):
		dp = 0
		min_temp = max_temp

	for j in range (0,len(planet)):


#		if dp == 0.0:
#			val = 1
#		else:
#			val = (dp + planet[j][b_i]-min_temp)/(dp + max_temp-min_temp)

#		print(val,(dp + planet[j][b_i]-min_temp),(dp + max_temp-min_temp))

		val = (planet[j][b_i]-min_temp)/(max_temp-min_temp)

		c = mycmap(val)

		n = planet[j]

		r1 = n[13]*scale_planet
		r2 = n[14]*scale_planet
		radii = [r1,r2]

		thetas = np.linspace(n[10],n[11],100)

		xs = np.outer(radii, np.cos(thetas)) + planet_cen[0]
		ys = np.outer(radii, np.sin(thetas)) + planet_cen[1]

		xs[1,:] = xs[1,::-1]
		ys[1,:] = ys[1,::-1]

		ax.fill(np.ravel(xs), np.ravel(ys), edgecolor=c,color=c,zorder=2)



	ax.set_facecolor(bg)

	if show_axes == False:
		ax.spines['bottom'].set_color(bg)
		ax.spines['left'].set_color(bg)
		ax.spines['top'].set_color(bg)
		ax.spines['right'].set_color(bg)
	else:
		ax.spines['bottom'].set_color(tc)
		ax.spines['left'].set_color(tc)
		ax.spines['top'].set_color(tc)
		ax.spines['right'].set_color(tc)

	ax.set(aspect=1)

	bs = 1.1
	ax.set_xlim(+bs,-bs)
	ax.set_ylim(-bs,+bs)

	if new_ax == True:
		bs = 1.1
		ax.set_xlim(+bs,-bs)
		ax.set_ylim(-bs,+bs)

	if show_axes == False:
		ax.get_xaxis().set_visible(False)
		ax.get_yaxis().set_visible(False)
	else:
		ax.get_xaxis().set_visible(True)
		ax.get_yaxis().set_visible(True)

	divider = make_axes_locatable(ax)
	# Append axes to the right of ax, with 20% width of ax

#	zero_temp = min_temp - dp
	zero_temp = min_temp
	data = [np.linspace(zero_temp,max_temp,1000)]*2
	fake, fake_ax = plt.subplots()
	mycax = fake_ax.imshow(data, interpolation='none', cmap=mycmap)
	plt.close(fake)

	if show_cax == True:
		cax = divider.append_axes("right", size="20%", pad=0.05)
	#	cbar = plt.colorbar(mycax, cax=cax,ticks=[1100,1300,1500,1700,1900])
		cbar = plt.colorbar(mycax, cax=cax)
		cbar.ax.tick_params(colors=tc)

		if temp_map == True:
			cbar.set_label('T (K)',color=tc)  # horizontal colorbar
		else:
			cbar.set_label('Relative brightness',color=tc)  # horizontal colorbar

	return ax

def make_movie():

	exp_time = 103.129

	tc = 200.0
	per = 0.81347753
	a = 0.01526
	inc = 82.33
	ecc = 0.0
	omega = 90
	a_rs = 4.855
	rp = 0.15938366414961666

	T_n = 1384.463713618203
	delta_T = 495.64620450883035
	xi = 0.30020002792596134

	T_n = 1127.8096575845159
	delta_T = 941.89297550917024
	xi = 0.64556829219408363

	u1 = 0
	u2 = 0

	T_s = 4520

	n_slice = 5

	n_ts = 100

	ts = np.linspace(tc-per,tc,n_ts)

	ps = ((ts-tc)/per)

	data = np.loadtxt('WASP43_HST13467/lc_cor_out.txt')

	ps_corr = data[:,0]
	lc_corr = data[:,1]

	for i in range(0,len(ps)):
		if(ps[i] > 1):
			ps[i] = ps[i] - np.floor(ps[i])
		if(ps[i] <= 0):
			ps[i] = ps[i] - np.ceil(ps[i]) + 1
		if(ps[i] == 1):
			ps[0] = 0

	pc = web.lightcurve(n_slice,ts,tc,per,a,inc,ecc,omega,a_rs,rp,xi,T_n,delta_T,u1,u2,T_s)

	bat_params = batman.TransitParams()
	bat_params.t0 = tc
	bat_params.t_secondary = tc+per/2.0
	bat_params.per = per
	bat_params.rp = rp
	bat_params.a = a_rs
	bat_params.inc = inc
	bat_params.ecc = ecc
	bat_params.w = omega
	bat_params.u = [0.4, 0]
	bat_params.limb_dark = "quadratic"		#FIXME - specify this value in one of the config files

	m = batman.TransitModel(bat_params, ts, supersample_factor=3, exp_time = exp_time/24./60./60.)

	lc = pc + (m.light_curve(bat_params)-1.0)

	phase_c = []
	phases = []

	ratio = 1/rp
	star_r = ratio

	dp = (T_n*0.1)
	min_temp = T_n
	max_temp = (T_n + delta_T)


	star_r_pix = 180
	star_offset_pix = 200

	p_imrat = star_r_pix*rp

	fn = 'sun_transp.png'
	image = read_png(fn)

#	zero_temp = min_temp - dp
	zero_temp = min_temp

	data = [np.linspace(zero_temp,max_temp,1000)]*2

	cax = plt.imshow(data, interpolation='none', cmap=plt.get_cmap('inferno'))
	plt.close()

	for i in range(0,n_ts-1):
		t = ts[i]
		coords = web.separation_of_centers(t,tc,per,a,inc,ecc,omega,a_rs,ratio)

		star_x = 0.0-coords[0]
		star_y = 0.0-coords[1]
		star_z = 0.0-coords[2]

		ni = str(i)

		if len(ni) < 2:
			ni = '0'+ni

		plotname = 'plots/blooper_'+'{:03d}'.format(i)+'.png'

		phase = ((t-tc)/per)

		if(phase > 1):
			phase = phase - np.floor(phase)
		if(phase < 0):
			phase = phase + np.ceil(phase) + 1

		lambda0 = (np.pi + phase*2*np.pi)
		phi0 = np.arctan2(star_y,star_z)
		if(lambda0 > 2*np.pi):
			lambda0 = lambda0 - 2*np.pi;
		if(lambda0 < -2*np.pi):
			lambda0 = lambda0 + 2*np.pi;


		phases += [phase]

		planet = np.array(web.generate_planet(n_slice,xi,T_n,delta_T,lambda0,phi0,u1,u2))

		ax1 = plt.subplot(211, aspect='equal')

		gs = gridspec.GridSpec(2, 2, width_ratios=[3, 1]) 

#		with sb.axes_style("ticks"):
#			ax2 = plt.subplot(gs[2])
#			sb.despine()

		ax3 = plt.subplot(gs[3], aspect='equal')

		phase_c += [np.sum(planet[:,16])]

		thetas = np.linspace(planet[0][10],planet[0][11],100)
		r = planet[0][14]
		radii = [0,r]
		xs = star_offset_pix+ p_imrat*np.outer(radii, np.cos(thetas)) + coords[0]*p_imrat
		ys = star_offset_pix+ p_imrat*np.outer(radii, np.sin(thetas)) + coords[1]*p_imrat
		xs[1,:] = xs[1,::-1]
		ys[1,:] = ys[1,::-1]

		val = (dp + planet[0][17]-min_temp)/(dp + max_temp-min_temp)

		c = plt.get_cmap('inferno')(val)
		ax1.fill(np.ravel(xs), np.ravel(ys), edgecolor=c,color=c,zorder=2)
		ax3.fill(np.ravel(xs), np.ravel(ys), edgecolor=c,color=c,zorder=2)

		for j in range (1,len(planet)):

			val = (dp + planet[j][17]-min_temp)/(dp + max_temp-min_temp)
			c = plt.get_cmap('inferno')(val)

			n = planet[j]

			r1 = n[13]
			r2 = n[14]
			radii = [r1,r2]

			thetas = np.linspace(n[10],n[11],100)

			xs = star_offset_pix+ p_imrat*np.outer(radii, np.cos(thetas)) - coords[0]*p_imrat
			ys = star_offset_pix+ p_imrat*np.outer(radii, np.sin(thetas)) - coords[1]*p_imrat

			xs[1,:] = xs[1,::-1]
			ys[1,:] = ys[1,::-1]

			ax1.fill(np.ravel(xs), np.ravel(ys), edgecolor=c,color=c,zorder=2)
			ax3.fill(np.ravel(xs), np.ravel(ys), edgecolor=c,color=c,zorder=2)


		thetas = np.linspace(planet[0][10],planet[0][11],100)
		radii = [0,star_r]
		xs = np.outer(radii, np.cos(thetas))
		ys = np.outer(radii, np.sin(thetas))
		xs[1,:] = xs[1,::-1]
		ys[1,:] = ys[1,::-1]
		c = plt.get_cmap('inferno')(1.0)

		if(abs(abs(phase)-0.5) > 0.25):
			#in front 
			s_zorder = 1
		else:
			s_zorder = 3

		ax1.set_facecolor('black')

		ax1.set_xlim(star_offset_pix+40*p_imrat,star_offset_pix+-40*p_imrat)
		ax1.set_ylim(star_offset_pix+-10*p_imrat,star_offset_pix+10*p_imrat)

		ax1.get_xaxis().set_visible(False)
		ax1.get_yaxis().set_visible(False)

		im = ax1.imshow(image,zorder=s_zorder)

		patch = patches.Circle((200, 200), radius=200, transform=ax1.transData)
		im.set_clip_path(patch)

		ax1.set_facecolor('black')
		ax2.set_facecolor('black')
		ax3.set_facecolor('black')

		ax1.spines['bottom'].set_color("black")
		ax1.spines['left'].set_color("black")
		ax3.spines['bottom'].set_color("black")
		ax3.spines['left'].set_color("black")

		ax2.spines['bottom'].set_color("#04d9ff")
		ax2.spines['left'].set_color("#04d9ff")

		ax2.xaxis.label.set_color("#04d9ff")
		ax2.yaxis.label.set_color("#04d9ff")

		ax2.plot(ps[:(i+1)],lc[:(i+1)],lw=2,color=("#04d9ff"))

		ax2.plot(ps_corr,lc_corr,',',color=("#04d9ff"))

	#	ax2.set_xlim(ps[0],ps[-1])
	#	ax2.set_ylim(np.median(lc)+1.1*(np.min(lc) - np.median(lc)),np.median(lc)+1.1*(np.max(lc) - np.median(lc)))

		ax2.set_xlim(0-0.02,1+0.02)

		ax2.set_ylabel('Relative flux')
		ax2.set_xlabel('Phase')

		ax2.set_ylim(0.9995,1.0009)
		ax2.set_yticks([0.9996,0.9998,1.0000,1.0002,1.0004,1.0006,1.0008])
		ax2.set_yticklabels(['0.9996','0.9998','1.0000','1.0002','1.0004','1.0006','1.0008'])


		bs = 1.1*p_imrat

		p_xpos = star_offset_pix - coords[0]*p_imrat
		p_ypos = star_offset_pix - coords[1]*p_imrat

		ax3.set_xlim(p_xpos+bs,p_xpos-bs)
		ax3.set_ylim(p_ypos-bs,p_ypos+bs)

		ax3.get_xaxis().set_visible(False)
		ax3.get_yaxis().set_visible(False)

		im = ax1.imshow(image,zorder=s_zorder)


		fig = plt.gcf()

		divider3 = make_axes_locatable(ax3)
		# Append axes to the right of ax3, with 20% width of ax3

		cax3 = divider3.append_axes("right", size="20%", pad=0.05)

		cbar3 = plt.colorbar(cax, cax=cax3,ticks=[1100,1300,1500,1700,1900])

		cbar3.ax.tick_params(colors=("#04d9ff"))

		cbar3.set_label('T (K)',color=("#04d9ff"))  # horizontal colorbar

		ax2.tick_params(axis='x', colors=("#04d9ff"))
		ax2.tick_params(axis='y', colors=("#04d9ff"))

		plt.tight_layout(pad=1.0)

		plt.savefig(plotname, bbox_inches='tight',pad_inches = 0.1,facecolor='black')

	#	plt.show()

		plt.close()



import spiderman as sp
import numpy as np

def load_grid(fname,layer):
	longitude, latitude, T = open_PT_profile(fname,layer)
	grid = format_grid(longitude,latitude,T)
	return grid

def open_PT_profile(fname,layer):
	layers = []
	for line in open(fname):
		try:
			layers += [int(line.strip('\n').split(',')[0])]
		except:
			''
	layers = np.array(layers)
	nlayers = np.max(layers)

#	print(nlayers)
	longitude = []
	latitude = []
	T = []


	for line in open(fname):
		try:
			this_layer = int(line.strip('\n').split(',')[0])
			if this_layer == layer:
				this_grid = line.strip('\n').split(',')
				longitude += [float(this_grid[1])]
				latitude += [float(this_grid[2])]
				T += [float(this_grid[4])]
		except:
			''
	T = np.array(T)
	longitude = np.array(longitude)
	latitude = np.array(latitude)
	return(longitude, latitude, T)

def format_grid(longitude,latitude,temp):
	# puts the grid into a format that spiderman can use
	# including wrapping/reflecting the boundaries to allow smooth and continuous interpolation

	longitude = longitude.flatten()
	latitude = latitude.flatten()
	temp = temp.flatten()

	LO, LA = np.unique(longitude),np.unique(latitude)
	T = np.reshape(temp,(len(LO),len(LA)))

	for i in range(0,len(LO)):
		if LO[i] > 180:
			LO[i] = LO[i] - 360
		if LO[i] < -180:
			LO[i] = LO[i] + 360

	if len(LO) > 100:
		T = np.vstack((T[-5],T[-4],T[-3],T[-2],T[-1],T,T[0],T[1],T[2],T[3],T[4]))
		LO = np.hstack((LO[-5:]-360,LO,LO[0:5]+360))
	else:
		T = np.vstack((T[-2],T[-1],T,T[0],T[1]))
		LO = np.hstack((LO[-2:]-360,LO,LO[0:2]+360))

	if len(LA) > 100:
		T = np.vstack((T[:,4],T[:,3],T[:,2],T[:,1],T[:,0],T.T,T[:,-1],T[:,-2],T[:,-3],T[:,-4],T[:,-5])).T
		LA = np.hstack((-180 - LA[0:5][::-1],LA,180-LA[-5:][::-1]))
	else:
		T = np.vstack((T[:,1],T[:,0],T.T,T[:,-1],T[:,-2])).T
		LA = np.hstack((-180 - LA[0:2][::-1],LA,180-LA[-2:][::-1]))

	grid = np.array([LO,LA,T])

	return grid


# -*- coding: utf-8 -*-
import os.path

use_fitsio = False
try:
    import fitsio
    use_fitsio = True
except:
    from astropy.io import fits
    use_fitsio = False

import matplotlib.pyplot as plt
import numpy as np
import spiderman
from scipy.interpolate import interp1d

def gen_grid(l1,l2,logg=4.5, response = False, stellar_model = "blackbody", verbose=False):
        #options for stellar models are "blackbody", "PHOENIX", and "path_to_model"
    z = -0.0

    h =6.62607004e-34;          #m^2/kg/s
    c =299792458.0;             #m/s
    kb =1.38064852e-23;         #m^2 kg /s^2 K

    teffs = [2500,3000,3500,4000,4500,5000,5500,6000,6500,7000]

    warned = False

    filter = response
    if response != False:
        filter = spiderman.get_filter(response)

    totals = []
    for teff in teffs:
        if stellar_model == "PHOENIX":
            if spiderman.rcParams.read == False: print('Add path to PHOENIX models to .spidermanrc file')
            wvl, flux = get_phoenix_spectra(teff,logg,z)
            PHOENIX_DIR = spiderman.rcParams['PHOENIX_DIR']

            if warned == False:
                print('using stellar spectra in '+PHOENIX_DIR)

            if ( ((l1 > np.min(wvl)) & (l1 < np.max(wvl))) & ((l2 > np.min(wvl)) & (l2 < np.max(wvl) )) ):
                totals += [sum_flux(wvl,flux,l1,l2,filter)]
            else:
                if warned == False:
                    print('wavelengths out of bound for stellar model, using blackbody approximation')
                b_wvl = np.linspace(l1,l2,1000)
                b_flux = (2.0*h*(c**2)/(b_wvl**5))*(1.0/( np.exp( (h*c)/(b_wvl*kb*teff) )- 1.0));
                totals += [sum_flux(b_wvl,b_flux,l1,l2,filter)]
        elif stellar_model == "blackbody":
            if warned == False:
                print('no stellar models provided, using blackbody approximation')
            b_wvl = np.linspace(l1,l2,1000)
            b_flux = (2.0*h*(c**2)/(b_wvl**5))*(1.0/( np.exp( (h*c)/(b_wvl*kb*teff) )- 1.0));       #SI units: W/sr/m^3
            totals += [sum_flux(b_wvl,b_flux,l1,l2,filter)]
        else:
            if os.path.isfile(stellar_model):
                spectrum = np.genfromtxt(stellar_model)
                wvl, flux = spectrum[:,0], spectrum[:,1] 
            else: print("Model stellar spectrum file", stellar_model, "not found")

            if ( ((l1 > np.min(wvl)) & (l1 < np.max(wvl))) & ((l2 > np.min(wvl)) & (l2 < np.max(wvl) )) ):
                totals += [sum_flux(wvl,flux,l1,l2,filter)]
            else:                    
                if warned == False:
                    print('wavelengths out of bound for stellar model, using blackbody approximation')
                b_wvl = np.linspace(l1,l2,1000)
                b_flux = (2.0*h*(c**2)/(b_wvl**5))*(1.0/( np.exp( (h*c)/(b_wvl*kb*teff) )- 1.0));
                totals += [sum_flux(b_wvl,b_flux,l1,l2,filter)]
        warned = True


    teffs = np.array(teffs)
    totals = np.array(totals)

    return [teffs, totals]

def sum_flux(wvl,flux,l1,l2,filter=False):


    mask = [(wvl > l1) & (wvl < l2)]

    diff = np.diff(wvl)
    diff = (np.append(diff,diff[-1:]) + np.append(diff[1:],diff[-2:]))/2

    diff = diff[mask]

    wvl = wvl[mask]
    flux = flux[mask]

    if filter != False:
        f = interp1d(filter[0],filter[1],kind='linear',bounds_error=True,axis=0)
        r = f(wvl)
    else:
        r = np.array([1.0]*len(wvl))

    total = 0.0

    for i in range(0,len(wvl)):
        total += r[i]*flux[i]*diff[i]
    return total

def get_phoenix_spectra(teff,logg,z):
    ftemplate = 'lte{teff:05d}-{logg:4.2f}{z:+3.1f}.PHOENIX-ACES-AGSS-COND-2011-HiRes.fits'

    PHOENIX_DIR = spiderman.rcParams['PHOENIX_DIR']

    filename = os.path.join(PHOENIX_DIR,ftemplate.format(teff=teff,logg=logg,z=z))

    # changing to si, W / m^3 / str
    if use_fitsio:
        flux,h = fitsio.read(filename, ext=0, header=True)
    else:
        flux,h = fits.getdata(filename, ext=0, header=True)
    
    flux = flux*1e-7*1e6/(np.pi)

    crval = h['CRVAL1']
    cdelt = h['CDELT1']
    ctype = h['CTYPE1']

    if ctype == 'AWAV-LOG':
        wvl = (np.exp(crval + cdelt*np.arange(0,len(flux))))*1e-10
    else:
        print('ctype is not log! It  is {}'.format(ctype))

    return wvl, flux
